@@ -1,3 +1,4 @@
+"""Nbclassic read/write auth example."""
 from jupyter_server.auth import Authorizer
 
 
@@ -11,4 +12,4 @@ class ReadWriteOnly(Authorizer):
         return True
 
 
-c.ServerApp.authorizer_class = ReadWriteOnly
+c.ServerApp.authorizer_class = ReadWriteOnly  # type:ignore[name-defined]
