@@ -1,2 +1,2 @@
-"""Pytest configurtion."""
+"""Pytest configuration."""
 pytest_plugins = ["jupyter_server.pytest_plugin"]
