@@ -3,7 +3,11 @@ Utilities for file-based Contents/Checkpoints managers.
 """
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+
+from __future__ import annotations
+
 import errno
+import hashlib
 import os
 import shutil
 from base64 import decodebytes, encodebytes
@@ -13,8 +17,9 @@ from functools import partial
 import nbformat
 from anyio.to_thread import run_sync
 from tornado.web import HTTPError
-from traitlets import Bool
+from traitlets import Bool, Enum
 from traitlets.config import Configurable
+from traitlets.config.configurable import LoggingConfigurable
 
 from jupyter_server.utils import ApiPath, to_api_path, to_os_path
 
@@ -104,9 +109,9 @@ def atomic_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     if text:
         # Make sure that text files have Unix linefeeds by default
         kwargs.setdefault("newline", "\n")
-        fileobj = open(path, "w", encoding=encoding, **kwargs)
+        fileobj = open(path, "w", encoding=encoding, **kwargs)  # noqa: SIM115
     else:
-        fileobj = open(path, "wb", **kwargs)
+        fileobj = open(path, "wb", **kwargs)  # noqa: SIM115
 
     try:
         yield fileobj
@@ -152,9 +157,9 @@ def _simple_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     if text:
         # Make sure that text files have Unix linefeeds by default
         kwargs.setdefault("newline", "\n")
-        fileobj = open(path, "w", encoding=encoding, **kwargs)
+        fileobj = open(path, "w", encoding=encoding, **kwargs)  # noqa: SIM115
     else:
-        fileobj = open(path, "wb", **kwargs)
+        fileobj = open(path, "wb", **kwargs)  # noqa: SIM115
 
     try:
         yield fileobj
@@ -165,7 +170,7 @@ def _simple_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     fileobj.close()
 
 
-class FileManagerMixin(Configurable):
+class FileManagerMixin(LoggingConfigurable, Configurable):
     """
     Mixin for ContentsAPI classes that interact with the filesystem.
 
@@ -187,16 +192,22 @@ class FileManagerMixin(Configurable):
         True,
         config=True,
         help="""By default notebooks are saved on disk on a temporary file and then if succefully written, it replaces the old ones.
-      This procedure, namely 'atomic_writing', causes some bugs on file system whitout operation order enforcement (like some networked fs).
+      This procedure, namely 'atomic_writing', causes some bugs on file system without operation order enforcement (like some networked fs).
       If set to False, the new notebook is written directly on the old one which could fail (eg: full filesystem or quota )""",
     )
 
+    hash_algorithm = Enum(  # type: ignore[call-overload]
+        hashlib.algorithms_available,
+        default_value="sha256",
+        config=True,
+        help="Hash algorithm to use for file content, support by hashlib",
+    )
+
     @contextmanager
     def open(self, os_path, *args, **kwargs):
         """wrapper around io.open that turns permission errors into 403"""
-        with self.perm_to_403(os_path):
-            with open(os_path, *args, **kwargs) as f:
-                yield f
+        with self.perm_to_403(os_path), open(os_path, *args, **kwargs) as f:
+            yield f
 
     @contextmanager
     def atomic_writing(self, os_path, *args, **kwargs):
@@ -204,7 +215,7 @@ class FileManagerMixin(Configurable):
         Depending on flag 'use_atomic_writing', the wrapper perform an actual atomic writing or
         simply writes the file (whatever an old exists or not)"""
         with self.perm_to_403(os_path):
-            kwargs["log"] = self.log  # type:ignore
+            kwargs["log"] = self.log
             if self.use_atomic_writing:
                 with atomic_writing(os_path, *args, **kwargs) as f:
                     yield f
@@ -224,7 +235,7 @@ class FileManagerMixin(Configurable):
                 # but nobody should be doing that anyway.
                 if not os_path:
                     os_path = e.filename or "unknown file"
-                path = to_api_path(os_path, root=self.root_dir)  # type:ignore
+                path = to_api_path(os_path, root=self.root_dir)  # type:ignore[attr-defined]
                 raise HTTPError(403, "Permission denied: %s" % path) from e
             else:
                 raise
@@ -234,63 +245,82 @@ class FileManagerMixin(Configurable):
 
         like shutil.copy2, but log errors in copystat
         """
-        copy2_safe(src, dest, log=self.log)  # type:ignore
+        copy2_safe(src, dest, log=self.log)
 
     def _get_os_path(self, path):
         """Given an API path, return its file system path.
 
         Parameters
         ----------
-        path : string
+        path : str
             The relative API path to the named file.
 
         Returns
         -------
-        path : string
+        path : str
             Native, absolute OS path to for a file.
 
         Raises
         ------
         404: if path is outside root
         """
-        root = os.path.abspath(self.root_dir)  # type:ignore
+        self.log.debug("Reading path from disk: %s", path)
+        root = os.path.abspath(self.root_dir)  # type:ignore[attr-defined]
         # to_os_path is not safe if path starts with a drive, since os.path.join discards first part
         if os.path.splitdrive(path)[0]:
             raise HTTPError(404, "%s is not a relative API path" % path)
         os_path = to_os_path(ApiPath(path), root)
+        # validate os path
+        # e.g. "foo\0" raises ValueError: embedded null byte
+        try:
+            os.lstat(os_path)
+        except OSError:
+            # OSError could be FileNotFound, PermissionError, etc.
+            # those should raise (or not) elsewhere
+            pass
+        except ValueError:
+            raise HTTPError(404, f"{path} is not a valid path") from None
+
         if not (os.path.abspath(os_path) + os.path.sep).startswith(root):
             raise HTTPError(404, "%s is outside root contents directory" % path)
         return os_path
 
-    def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
+    def _read_notebook(
+        self, os_path, as_version=4, capture_validation_error=None, raw: bool = False
+    ):
         """Read a notebook from an os path."""
-        with self.open(os_path, "r", encoding="utf-8") as f:
-            try:
-                return nbformat.read(
-                    f, as_version=as_version, capture_validation_error=capture_validation_error
-                )
-            except Exception as e:
-                e_orig = e
+        answer = self._read_file(os_path, "text", raw=raw)
 
-            # If use_atomic_writing is enabled, we'll guess that it was also
-            # enabled when this notebook was written and look for a valid
-            # atomic intermediate.
-            tmp_path = path_to_intermediate(os_path)
-
-            if not self.use_atomic_writing or not os.path.exists(tmp_path):
-                raise HTTPError(
-                    400,
-                    f"Unreadable Notebook: {os_path} {e_orig!r}",
-                )
-
-            # Move the bad file aside, restore the intermediate, and try again.
-            invalid_file = path_to_invalid(os_path)
-            replace_file(os_path, invalid_file)
-            replace_file(tmp_path, os_path)
-            return self._read_notebook(
-                os_path, as_version, capture_validation_error=capture_validation_error
+        try:
+            nb = nbformat.reads(
+                answer[0],
+                as_version=as_version,
+                capture_validation_error=capture_validation_error,
             )
 
+            return (nb, answer[2]) if raw else nb  # type:ignore[misc]
+        except Exception as e:
+            e_orig = e
+
+        # If use_atomic_writing is enabled, we'll guess that it was also
+        # enabled when this notebook was written and look for a valid
+        # atomic intermediate.
+        tmp_path = path_to_intermediate(os_path)
+
+        if not self.use_atomic_writing or not os.path.exists(tmp_path):
+            raise HTTPError(
+                400,
+                f"Unreadable Notebook: {os_path} {e_orig!r}",
+            )
+
+        # Move the bad file aside, restore the intermediate, and try again.
+        invalid_file = path_to_invalid(os_path)
+        replace_file(os_path, invalid_file)
+        replace_file(tmp_path, os_path)
+        return self._read_notebook(
+            os_path, as_version, capture_validation_error=capture_validation_error, raw=raw
+        )
+
     def _save_notebook(self, os_path, nb, capture_validation_error=None):
         """Save a notebook to an os_path."""
         with self.atomic_writing(os_path, encoding="utf-8") as f:
@@ -301,14 +331,46 @@ class FileManagerMixin(Configurable):
                 capture_validation_error=capture_validation_error,
             )
 
-    def _read_file(self, os_path, format):
+    def _get_hash(self, byte_content: bytes) -> dict[str, str]:
+        """Compute the hash hexdigest for the provided bytes.
+
+        The hash algorithm is provided by the `hash_algorithm` attribute.
+
+        Parameters
+        ----------
+        byte_content : bytes
+            The bytes to hash
+
+        Returns
+        -------
+        A dictionary to be appended to a model {"hash": str, "hash_algorithm": str}.
+        """
+        algorithm = self.hash_algorithm
+        h = hashlib.new(algorithm)
+        h.update(byte_content)
+        return {"hash": h.hexdigest(), "hash_algorithm": algorithm}
+
+    def _read_file(
+        self, os_path: str, format: str | None, raw: bool = False
+    ) -> tuple[str | bytes, str] | tuple[str | bytes, str, bytes]:
         """Read a non-notebook file.
 
-        os_path: The path to be read.
-        format:
-          If 'text', the contents will be decoded as UTF-8.
-          If 'base64', the raw bytes contents will be encoded as base64.
-          If not specified, try to decode as UTF-8, and fall back to base64
+        Parameters
+        ----------
+        os_path: str
+            The path to be read.
+        format: str
+            If 'text', the contents will be decoded as UTF-8.
+            If 'base64', the raw bytes contents will be encoded as base64.
+            If 'byte', the raw bytes contents will be returned.
+            If not specified, try to decode as UTF-8, and fall back to base64
+        raw: bool
+            [Optional] If True, will return as third argument the raw bytes content
+
+        Returns
+        -------
+        (content, format, byte_content) It returns the content in the given format
+        as well as the raw byte content.
         """
         if not os.path.isfile(os_path):
             raise HTTPError(400, "Cannot read non-file %s" % os_path)
@@ -316,11 +378,22 @@ class FileManagerMixin(Configurable):
         with self.open(os_path, "rb") as f:
             bcontent = f.read()
 
+        if format == "byte":
+            # Not for http response but internal use
+            return (bcontent, "byte", bcontent) if raw else (bcontent, "byte")
+
         if format is None or format == "text":
             # Try to interpret as unicode if format is unknown or if unicode
             # was explicitly requested.
             try:
-                return bcontent.decode("utf8"), "text"
+                return (
+                    (bcontent.decode("utf8"), "text", bcontent)
+                    if raw
+                    else (
+                        bcontent.decode("utf8"),
+                        "text",
+                    )
+                )
             except UnicodeError as e:
                 if format == "text":
                     raise HTTPError(
@@ -328,7 +401,14 @@ class FileManagerMixin(Configurable):
                         "%s is not UTF-8 encoded" % os_path,
                         reason="bad format",
                     ) from e
-        return encodebytes(bcontent).decode("ascii"), "base64"
+        return (
+            (encodebytes(bcontent).decode("ascii"), "base64", bcontent)
+            if raw
+            else (
+                encodebytes(bcontent).decode("ascii"),
+                "base64",
+            )
+        )
 
     def _save_file(self, os_path, content, format):
         """Save content of a generic file."""
@@ -360,41 +440,47 @@ class AsyncFileManagerMixin(FileManagerMixin):
 
         like shutil.copy2, but log errors in copystat
         """
-        await async_copy2_safe(src, dest, log=self.log)  # type:ignore
+        await async_copy2_safe(src, dest, log=self.log)
 
-    async def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
+    async def _read_notebook(
+        self, os_path, as_version=4, capture_validation_error=None, raw: bool = False
+    ):
         """Read a notebook from an os path."""
-        with self.open(os_path, encoding="utf-8") as f:
-            try:
-                return await run_sync(
-                    partial(
-                        nbformat.read,
-                        as_version=as_version,
-                        capture_validation_error=capture_validation_error,
-                    ),
-                    f,
-                )
-            except Exception as e:
-                e_orig = e
+        answer = await self._read_file(os_path, "text", raw)
 
-            # If use_atomic_writing is enabled, we'll guess that it was also
-            # enabled when this notebook was written and look for a valid
-            # atomic intermediate.
-            tmp_path = path_to_intermediate(os_path)
-
-            if not self.use_atomic_writing or not os.path.exists(tmp_path):
-                raise HTTPError(
-                    400,
-                    f"Unreadable Notebook: {os_path} {e_orig!r}",
-                )
-
-            # Move the bad file aside, restore the intermediate, and try again.
-            invalid_file = path_to_invalid(os_path)
-            await async_replace_file(os_path, invalid_file)
-            await async_replace_file(tmp_path, os_path)
-            return await self._read_notebook(
-                os_path, as_version, capture_validation_error=capture_validation_error
+        try:
+            nb = await run_sync(
+                partial(
+                    nbformat.reads,
+                    as_version=as_version,
+                    capture_validation_error=capture_validation_error,
+                ),
+                answer[0],
             )
+            return (nb, answer[2]) if raw else nb  # type:ignore[misc]
+        except Exception as e:
+            e_orig = e
+
+        # If use_atomic_writing is enabled, we'll guess that it was also
+        # enabled when this notebook was written and look for a valid
+        # atomic intermediate.
+        tmp_path = path_to_intermediate(os_path)
+
+        if not self.use_atomic_writing or not os.path.exists(tmp_path):
+            raise HTTPError(
+                400,
+                f"Unreadable Notebook: {os_path} {e_orig!r}",
+            )
+
+        # Move the bad file aside, restore the intermediate, and try again.
+        invalid_file = path_to_invalid(os_path)
+        await async_replace_file(os_path, invalid_file)
+        await async_replace_file(tmp_path, os_path)
+        answer = await self._read_notebook(
+            os_path, as_version, capture_validation_error=capture_validation_error, raw=raw
+        )
+
+        return answer
 
     async def _save_notebook(self, os_path, nb, capture_validation_error=None):
         """Save a notebook to an os_path."""
@@ -409,14 +495,27 @@ class AsyncFileManagerMixin(FileManagerMixin):
                 f,
             )
 
-    async def _read_file(self, os_path, format):
+    async def _read_file(  # type: ignore[override]
+        self, os_path: str, format: str | None, raw: bool = False
+    ) -> tuple[str | bytes, str] | tuple[str | bytes, str, bytes]:
         """Read a non-notebook file.
 
-        os_path: The path to be read.
-        format:
-          If 'text', the contents will be decoded as UTF-8.
-          If 'base64', the raw bytes contents will be encoded as base64.
-          If not specified, try to decode as UTF-8, and fall back to base64
+        Parameters
+        ----------
+        os_path: str
+            The path to be read.
+        format: str
+            If 'text', the contents will be decoded as UTF-8.
+            If 'base64', the raw bytes contents will be encoded as base64.
+            If 'byte', the raw bytes contents will be returned.
+            If not specified, try to decode as UTF-8, and fall back to base64
+        raw: bool
+            [Optional] If True, will return as third argument the raw bytes content
+
+        Returns
+        -------
+        (content, format, byte_content) It returns the content in the given format
+        as well as the raw byte content.
         """
         if not os.path.isfile(os_path):
             raise HTTPError(400, "Cannot read non-file %s" % os_path)
@@ -424,11 +523,22 @@ class AsyncFileManagerMixin(FileManagerMixin):
         with self.open(os_path, "rb") as f:
             bcontent = await run_sync(f.read)
 
+        if format == "byte":
+            # Not for http response but internal use
+            return (bcontent, "byte", bcontent) if raw else (bcontent, "byte")
+
         if format is None or format == "text":
             # Try to interpret as unicode if format is unknown or if unicode
             # was explicitly requested.
             try:
-                return bcontent.decode("utf8"), "text"
+                return (
+                    (bcontent.decode("utf8"), "text", bcontent)
+                    if raw
+                    else (
+                        bcontent.decode("utf8"),
+                        "text",
+                    )
+                )
             except UnicodeError as e:
                 if format == "text":
                     raise HTTPError(
@@ -436,7 +546,11 @@ class AsyncFileManagerMixin(FileManagerMixin):
                         "%s is not UTF-8 encoded" % os_path,
                         reason="bad format",
                     ) from e
-        return encodebytes(bcontent).decode("ascii"), "base64"
+        return (
+            (encodebytes(bcontent).decode("ascii"), "base64", bcontent)
+            if raw
+            else (encodebytes(bcontent).decode("ascii"), "base64")
+        )
 
     async def _save_file(self, os_path, content, format):
         """Save content of a generic file."""
