@@ -12,6 +12,12 @@ from traitlets.config import Config
 CULL_TIMEOUT = 30 if platform.python_implementation() == "PyPy" else 5
 CULL_INTERVAL = 1
 
+sample_kernel_json_with_metadata = {
+    "argv": ["cat", "{connection_file}"],
+    "display_name": "Test kernel",
+    "metadata": {"cull_idle_timeout": 0},
+}
+
 
 @pytest.fixture(autouse=True)
 def suppress_deprecation_warnings():
@@ -24,6 +30,21 @@ def suppress_deprecation_warnings():
         yield
 
 
+@pytest.fixture()
+def jp_kernelspec_with_metadata(jp_data_dir):
+    """Configures some sample kernelspecs in the Jupyter data directory."""
+    kenrel_spec_name = "sample_with_metadata"
+    sample_kernel_dir = jp_data_dir.joinpath("kernels", kenrel_spec_name)
+    sample_kernel_dir.mkdir(parents=True)
+    # Create kernel json file
+    sample_kernel_file = sample_kernel_dir.joinpath("kernel.json")
+    kernel_json = sample_kernel_json_with_metadata.copy()
+    sample_kernel_file.write_text(json.dumps(kernel_json))
+    # Create resources text
+    sample_kernel_resources = sample_kernel_dir.joinpath("resource.txt")
+    sample_kernel_resources.write_text("resource")
+
+
 @pytest.mark.parametrize(
     "jp_server_config",
     [
@@ -73,6 +94,24 @@ async def test_cull_idle(jp_fetch, jp_ws_fetch):
     assert culled
 
 
+async def test_cull_idle_disable(jp_fetch, jp_ws_fetch, jp_kernelspec_with_metadata):
+    r = await jp_fetch("api", "kernels", method="POST", allow_nonstandard_methods=True)
+    kernel = json.loads(r.body.decode())
+    kid = kernel["id"]
+
+    # Open a websocket connection.
+    ws = await jp_ws_fetch("api", "kernels", kid, "channels")
+
+    r = await jp_fetch("api", "kernels", kid, method="GET")
+    model = json.loads(r.body.decode())
+    assert model["connections"] == 1
+    culled = await get_cull_status(kid, jp_fetch)  # connected, should not be culled
+    assert not culled
+    ws.close()
+    culled = await get_cull_status(kid, jp_fetch)  # not connected, should not be culled
+    assert not culled
+
+
 # Pending kernels was released in Jupyter Client 7.1
 # It is currently broken on Windows (Jan 2022). When fixed, we can remove the Windows check.
 # See https://github.com/jupyter-server/jupyter_server/issues/672
